local AutoRoller = LibStub('AceAddon-3.0'):NewAddon('AutoRoller', 'AceEvent-3.0', 'AceConsole-3.0')
_G.AutoRoller = AutoRoller

local L = LibStub('AceLocale-3.0'):GetLocale('AutoRoller', true)

local ipairs = ipairs
local ConfirmLootRoll = ConfirmLootRoll
local StaticPopup_Hide = StaticPopup_Hide
local RollOnLoot = RollOnLoot
local format = format
local print = print
local GetLootRollItemLink = GetLootRollItemLink
local GetLootRollItemInfo = GetLootRollItemInfo
local GetItemInfo = GetItemInfo
local GetNumRaidMembers = GetNumRaidMembers
local InCombatLockdown = InCombatLockdown
local GetZoneText = GetZoneText
local GetSubZoneText = GetSubZoneText
local setfenv = setfenv
local pcall = pcall
local tonumber = tonumber
local loadstring = loadstring
local strmatch = strmatch

local ADDON_DISABLED = ADDON_DISABLED
local ENABLE = ENABLE
local LOOT_ROLL_GREED = LOOT_ROLL_GREED
local LOOT_ROLL_NEED = LOOT_ROLL_NEED
local LOOT_ROLL_PASSED = LOOT_ROLL_PASSED
local LOOT_ROLL_DISENCHANT = LOOT_ROLL_DISENCHANT


---------------   initialize   ---------------

-- action := {0 : pass, 1 : need, 2 : greed, 3 : nothing, 4 : disenchant}

local defaults = {
   char = {
      enabled = true,
      chat = true,
      disableBoPConfirm = false,
   },
   profile = {
      filter = [[
/greed boe
/none bop
/none itype == 'Recipe'
      ]],
   },
}

function AutoRoller:OnInitialize()
   local db = LibStub('AceDB-3.0'):New('AutoRollerDB', defaults, 'Default')
   self.db = db
   local RegisterCallback = db.RegisterCallback
   RegisterCallback(self, 'OnProfileChanged', 'OnEnable')
   RegisterCallback(self, 'OnProfileCopied', 'OnEnable')
   RegisterCallback(self, 'OnProfileReset', 'OnEnable')

   self:RegisterOptions()
end

function AutoRoller:OnEnable()
   local db = self.db
   local profile = db.profile
   local char = db.char

   --covert old options
   --here instead of OnInitialize to get OnProfileChanged

   if profile.rules then
      local rules = profile.rules

      local newrule = ''
      local line = '%s/%s %s\n'
      local rollType = {'pass', 'need', 'greed', 'none', 'disenchant'} --, [0] = 'pass'}
      for i,rule in pairs(rules) do

         --first three rule.eabled wasnt saved because same as defaults
         if rule and i <= 3 and rule.enabled == nil then
            rule.enabled = true
         end

         local newval = format(line, rule.enabled and '' or '--', rollType[rule.action + 1], rule.logic)
         newrule = newrule .. newval
      end
      profile.filter = newrule
      profile.rules, profile.enabled, profile.disableBoPConfirm = nil
   end

   --

   self:UpdateSize(profile.filter) -- update size of filters input box

   if not char.enabled then
      return self:Disable()
   end

   self:RegisterEvent('CONFIRM_LOOT_ROLL')
   self:RegisterEvent('CONFIRM_DISENCHANT_ROLL', 'CONFIRM_LOOT_ROLL') -- disenchant uses same popup as regular confirm loot
   self:RegisterEvent('START_LOOT_ROLL')
end

function AutoRoller:OnDisable()
end

---------------   Core   ---------------

local tags = {
   --hard coded values
   poor = 0,
   common = 1,
   uncommon = 2,
   rare = 3,
   epic = 4,
   legendary = 5,
   artifact = 6,
   heirloom = 7,
   silver = 100,
   gold = 10000,

   --mutable per item
   raid = false, --(GetNumRaidMembers() > 0),
   party = true, --not raid
   combat = false, --InCombatLockdown()
   quality = 2,
   itype = 'Armor', --'Armor', 'Consumable', 'Container', 'Gem', 'Key', 'Miscellaneous', 'Money', 'Reagent', 'Recipe', 'Projectile', 'Quest', 'Quiver', 'Trade Goods', 'Weapon'
   subtype = 'Plate', --http://www.wowwiki.com/ItemType
   ilvl = 213,
   reqlvl = 80,
   name = 'Item of Lewting',
   id = 123456,
   boe = true,
   bop = false,
   zone = 'Ulduar',
   subzone = 'Expedition Base Camp',
   value = 500, -- item vendor value in copper (1s = 100c, 1g = 10000c)

   green = false,
   blue = true,
   purple = false,

   need = true,
   greed = true,
   disenchant = true,

   betterilvl = true,

   gearscore = 100,
   bettergs = true,
}
AutoRoller.tags = tags

setmetatable(tags, {__index = _G})

local memfunc, memerror = {}, {}
--~ setmetatable(mem, {__mode = 'kv'}) -- shouldnt change that much to need decay
local memoizedloadstring =  function(str)
   local func, error = memfunc[str], memerror[str]
   if not func then
      func, error = loadstring(str)
      memfunc[str], memerror[str] = func, error
   end
   return func, error
end

local function pcallLogic(logic)
   local func, error = memoizedloadstring('return '..logic)
   if not func then
      return false, error
   end
   setfenv(func, tags)
   return pcall(func)
end

--returns an iterator of disabled,action,filter from filter string
local gmatch = string.gmatch
local function splitFilter(filter)
   return gmatch(filter..'\n', '(%--) -/(%w+) +(.-)\n')
end

local actionTolRoll = {
   none = -1,
   pass = 0,
   need = 1,
   greed = 2,
   disenchant = 3,
}

--takes in filter string and canNeed
--assume filter is pefectly formed and each line is separated by \n
local function EvaluateFilters(filters, canNeed)
   for disabled, action, filter in splitFilter(filters) do
      if disabled == '' and (canNeed or action ~= 'need') then
         local success,val = pcallLogic(filter)
         if success and val then
            return actionTolRoll[action]
         end
      end
   end

   return -1
end

function AutoRoller:ValidateFilter(info, value)
   for disabled, action, filter in splitFilter(value) do
      if not actionTolRoll[action] then
         local error = format(ERR_UNKNOWN_MACRO_OPTION_S, action)
         print(error)
         return error
      end

      local success, error = pcallLogic(filter)
      if not success then
         print(error)
         return error
      end
   end

   return true -- no errors
end

local function GetItemID(link)
   return tonumber(strmatch(link, 'item:(.-):'))
end

-- Suggested by Warloxx

local EquipLocations = {
   INVTYPE_AMMO = {0},
   INVTYPE_HEAD = {1},
   INVTYPE_NECK = {2},
   INVTYPE_SHOULDER = {3},
   INVTYPE_BODY = {4},
   INVTYPE_CHEST = {5},
   INVTYPE_ROBE = {5},
   INVTYPE_WAIST = {6},
   INVTYPE_LEGS = {7},
   INVTYPE_FEET = {8},
   INVTYPE_WRIST = {9},
   INVTYPE_HAND = {10},
   INVTYPE_FINGER = {11, 12},
   INVTYPE_TRINKET = {13, 14},
   INVTYPE_CLOAK = {15},
   INVTYPE_WEAPON = {16, 17},
   INVTYPE_SHIELD = {17},
   INVTYPE_2HWEAPON = {16, 17},
   INVTYPE_WEAPONMAINHAND = {16},
   INVTYPE_WEAPONOFFHAND = {17},
   INVTYPE_HOLDABLE = {17},
   INVTYPE_RANGED = {18},
   INVTYPE_THROWN = {18},
   INVTYPE_RANGEDRIGHT = {18},
   INVTYPE_RELIC = {18},
   INVTYPE_TABARD = {19},
}

local function compareilvl(loc, currentilvl)
   for i,slot in ipairs(EquipLocations[loc] or {}) do
      local link = GetInventoryItemLink('player', slot)
      if not link or select(4, GetItemInfo(link)) >= currentilvl then return true end
   end

   return false
end

local function comparegs(loc, currentgs)
   for i,slot in ipairs(EquipLocations[loc] or {}) do
      local link = GetInventoryItemLink('player', slot)
      if not link or GearScore_GetItemScore(link) >= currentgs then return true end
   end

   return false
end

function AutoRoller:START_LOOT_ROLL(event, rollID)
   local db = self.db
   local char = db.char
   local filter = db.profile.filter

   if not char.enabled then return end

   local link,_ = GetLootRollItemLink(rollID)

   local _, name, _, quality, bop, canNeed, canGreed, canDisenchant = GetLootRollItemInfo(rollID)
   local _,_,_, ilvl, reqlvl,  itype, subtype, equiploc,_,_, value = GetItemInfo(link) -- cant use name (item must be in inventory)

   local raid = GetNumRaidMembers() > 0

   tags.name = name
   tags.quality = quality
   tags.bop = bop
   tags.ilvl = ilvl
   tags.reqlvl = reqlvl
   tags.itype = itype
   tags.subtype = subtype
   tags.value = value
   tags.raid = raid
   tags.party = not raid
   tags.combat = InCombatLockdown()
   tags.id = GetItemID(link)
   tags.boe = not tags.bop
   tags.zone, tags.subzone = GetZoneText(), GetSubZoneText()

   tags.green = quality == 2
   tags.blue = quality == 3
   tags.purple = quality == 4

   tags.need = canNeed
   tags.greed = canGreed
   tags.disenchant = canDisenchant

   tags.betterilvl = compareilvl(equiploc, ilvl)

   if GearScore_GetItemScore then
      local gs = GearScore_GetItemScore(link)
      tags.gearscore = gs
      tags.bettergs = comparegs(equiploc, gs)
   end

   local roll = EvaluateFilters(filter, canNeed)
   if roll == -1 then return end

   local msg
   if roll == 0 then
      msg = LOOT_ROLL_PASSED
   elseif roll == 1 then
      msg = LOOT_ROLL_NEED
   elseif roll == 3 and canDisenchant then
      msg = LOOT_ROLL_DISENCHANT
   else --can't disenchant or greed selected, so greed
      if not canGreed then return end
      roll = 2
      msg = LOOT_ROLL_GREED
   end

   if char.chat then
      print(format(msg, "|cffffd200AutoRoller|r", link))
   end

   --0 to pass, 1 to Need an item, 2 to Greed an item, and 3 to Disenchant an item.
   RollOnLoot(rollID, roll)
end

function AutoRoller:CONFIRM_LOOT_ROLL(event, id, roll)
   if self.db.char.disableBoPConfirm then
      ConfirmLootRoll(id, roll)
      StaticPopup_Hide('CONFIRM_LOOT_ROLL') --may not be necessary
   end
end

local function RedGreen(enabled)
   if enabled then
      return 0,1,0
   end
   return 1,0,0
end

AutoRoller.dataobj = LibStub:GetLibrary('LibDataBroker-1.1'):NewDataObject('AutoRoller', {
   type = 'launcher',
   icon = 'Interface\\Buttons\\UI-GroupLoot-Dice-Up',
   tocname = 'AutoRoller',
   label = 'AutoRoller',
   OnTooltipShow = function(tooltip)
      tooltip:AddLine('AutoRoller')

      local db = AutoRoller.db
      local char = db.char

      local AddLine = tooltip.AddLine

      local enabled = char.enabled
      AddLine(tooltip, enabled and ENABLE or ADDON_DISABLED, RedGreen(enabled))

      enabled = char.chat
      AddLine(tooltip, CHAT, RedGreen(enabled))

      enabled = char.disableBoPConfirm
      AddLine(tooltip, L["Hide BoP Confirmation"], RedGreen(enabled))

      AddLine(tooltip, db.profile.filter)
   end,
})
