local AutoRoller = AutoRoller
local L = LibStub('AceLocale-3.0'):GetLocale('AutoRoller', true)

function AutoRoller.UpdateSize(handler, value)
	local _, count = value:gsub("\n", "\n")
	handler.optionTable.args[FILTERS].multiline = count + 5
end

local function GetProfileValue(info)
	local value = info.handler.db.profile[info.arg]
	info.handler:UpdateSize(value)
   return value
end

local function SetProfileValue(info, value)
   info.handler.db.profile[info.arg] = value
	info.handler:UpdateSize(value)
end

local function GetCharValue(info)
   return info.handler.db.char[info.arg]
end

local function SetCharValue(info, value)
   info.handler.db.char[info.arg] = value
end

local function SetEnabled(info, v)
   local self = info.handler
   self.db.char.enabled = v
   if v then
      self:Enable()
   else
      self:Disable()
   end
end

local constantString = ""
do --constants
	local constant = {'poor', 'common', 'uncommon', 'rare', 'epic', 'legendary', 'artifact', 'heirloom', 'silver', 'gold'}
	local constantPattern = '%s   |cffEF9C00%s|r   |cff00FFFF%d|r\n'
	for i = 0, 7 do
		constantString = format(constantPattern, constantString, constant[i+1], i)
	end
	constantString = format(constantPattern, constantString, constant[9], 100)
	constantString = format(constantPattern, constantString, constant[10], 10000)
end

local variablePattern = '%s   |cffEF9C00%s|r   |cff00FFFF%s|r   %s\n' -- tag type description

local variableString = ""
do --variables
	local variable = {'raid', 'party', 'combat', 'quality', 'itype', 'subtype', 'ilvl', 'reqlvl', 'name', 'id', 'boe', 'bop',
							 'zone', 'subzone', 'value', 'green', 'blue', 'purple', 'need', 'greed', 'disenchant', 'betterilvl'}
	for i,tag in ipairs(variable) do
		variableString = format(variablePattern, variableString, tag, type(AutoRoller.tags[tag]), L[tag])
	end
end

local optionalString = ""
do
	local optional = {'gearscore', 'bettergs'}
	for i,tag in ipairs(optional) do
		optionalString = format(variablePattern, optionalString, tag, type(AutoRoller.tags[tag]), L[tag])
	end
end


AutoRoller.optionTable = {
   name = 'AutoRoller',
   desc = L["Automatically roll on items."],
   type = 'group',
   handler = AutoRoller,
   get = GetCharValue,
   set = SetCharValue,
   args = {
      desc = {
         name = L["Automatically roll on items."],
         type = 'description',
         order = 100,
         width = 'full',
      },
      [ENABLE] = {
         name = ENABLE,
         type = 'toggle',
         order = 200,
         get = 'IsEnabled',
         width = 'full',
         set = SetEnabled,
      },
      [CHAT] = {
         name = CHAT_OPTIONS_LABEL,
         desc = L["Output to chat when automatically rolling on an item."],
         type = 'toggle',
         order = 300,
         arg = 'chat',
      },
      disableBoPConfirm = {
         name = L["Hide BoP Confirmation"],
         desc = L["Hide the confirmation dialog that appears every time you roll on a bind on pickup item."],
         type = 'toggle',
         order = 350,
         arg = 'disableBoPConfirm',
      },
		[FILTERS] = {
			name = FILTERS,
			type = 'input',
			order = 400,
			width = 'full',
			multiline = 10,
			arg = 'filter',
			get = GetProfileValue,
			set = SetProfileValue,
			validate = 'ValidateFilter',
		},
      [HELP_LABEL..1] = {
         name = L["tag description"],
         type = 'description',
         order = 500,
         width = 'full',
      },
      [HELP_LABEL..2] = {
         name = L["constant description"],
         type = 'description',
         order = 600,
         width = 'full',
      },
      [HELP_LABEL..3] = {
         name = constantString,
         type = 'description',
         order = 700,
         width = 'full',
      },
      [HELP_LABEL..4] = {
         name = L["variable description"],
         type = 'description',
         order = 800,
         width = 'full',
      },
      [HELP_LABEL..5] = {
         name = variableString,
         type = 'description',
         order = 900,
         width = 'full',
      },
      [HELP_LABEL..6] = {
         name = L["optional description"],
         type = 'description',
         order = 1000,
         width = 'full',
      },
      [HELP_LABEL..7] = {
         name = optionalString,
         type = 'description',
         order = 1100,
         width = 'full',
      },
      [HELP_LABEL..8] = {
         name = L["example-description"],
         type = 'description',
         order = 1200,
         width = 'full',
      },
      [HELP_LABEL..9] = {
         name = [[
   /none (subtype == "Pet") or (subtype == 'Mount')
   /need (name == "Battered Hilt")
   /disenchant green
   /greed (name == 'Frozen Orb') or not (need or disenchant)
   /disenchant party and blue and (value < (7 * gold)) and not betterilvl
   /greed party and blue and not betterilvl
]],
         type = 'description',
         order = 1300,
         width = 'full',
      },
   }
}

function AutoRoller:RegisterOptions()
   local options = self.optionTable
   local profile = LibStub('AceDBOptions-3.0'):GetOptionsTable(self.db)

   local registry = LibStub('AceConfigRegistry-3.0')
   local dialog = LibStub('AceConfigDialog-3.0')
   LibStub('AceConfig-3.0'):RegisterOptionsTable('AutoRoller', options, 'autoroller')

   registry:RegisterOptionsTable('AutoRoller Options', options)
   registry:RegisterOptionsTable('AutoRoller Profiles', profile)

   local main = dialog:AddToBlizOptions('AutoRoller Options', 'AutoRoller')
   dialog:AddToBlizOptions('AutoRoller Profiles', 'Profiles', 'AutoRoller')

   if self.dataobj then
      self.dataobj.OnClick = function() InterfaceOptionsFrame_OpenToCategory(main) end
   end
end
